from pathlib import Path
import os


def set_java_home():
    databricks_java_path = Path.home().joinpath(".databricks-connect-java")

    if not databricks_java_path.exists():
        raise Exception(f"Databricks Java (JDK) path {databricks_java_path} does not exist")

    os.environ["JAVA_HOME"] = str(databricks_java_path)



from box import Box
from pyspark.sql.session import SparkSession
from databricksbundle.storage.StorageConfiguratorInterface import StorageConfiguratorInterface


class TestingConfigurator(StorageConfiguratorInterface):
    def configure(self, spark: SparkSession, config: Box):
        spark.conf.set(f"testing.storage.{config.storage_name}", f"tenant/{config.tenant_id}")
        spark.conf.set(
            f"testing.secrets.{config.storage_name}.client_id", f"secrets/{config.client_id.secret_scope}/{config.client_id.secret_key}"
        )
        spark.conf.set(
            f"testing.secrets.{config.storage_name}.client_secret",
            f"secrets/{config.client_secret.secret_scope}/{config.client_secret.secret_key}",
        )

    def get_type(self):
        return "testing"



from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper
from databricksbundle.dbutils.IPythonDbUtilsResolver import resolve_dbutils


class DatabricksDbUtilsFactory:
    def create(self) -> DbUtilsWrapper:
        return DbUtilsWrapper(resolve_dbutils)



import pandas as pd
from daipecore.pandas.dataframe.PandasDataFrameShowMethodInterface import PandasDataFrameShowMethodInterface
from databricksbundle.notebook.helpers import ipython_display


class PandasDataFrameDisplay(PandasDataFrameShowMethodInterface):
    def show(self, df: pd.DataFrame):
        ipython_display(df)



from daipecore.widgets.Widgets import Widgets
from databricksbundle.detector import is_databricks
from databricksbundle.widgets.DatabricksWidgetsLabelGenerator import DatabricksWidgetsLabelGenerator
from pyspark.dbutils import DBUtils


class DatabricksWidgets(Widgets):
    def __init__(self, dbutils: DBUtils):
        self.__dbutils = dbutils
        self.__multiselect_fields = []
        self.__label_generator = DatabricksWidgetsLabelGenerator()

    def add_text(self, name: str, default_value: str = "", label: str = None):
        default_value = default_value if default_value is not None else ""
        label = self.__label_generator.generate_widget_label(name) if label is None else label

        self.__dbutils.widgets.text(name, default_value, label)

    def add_select(self, name: str, choices: list, default_value: str, label: str = None):
        if None in choices:
            raise Exception("Value None cannot be used as choice, use empty string instead")

        if default_value not in choices:
            raise Exception(f'Default value "{default_value}" not among choices')

        label = self.__label_generator.generate_widget_label(name) if label is None else label

        self.__dbutils.widgets.dropdown(name, default_value, choices, label)

    def add_multiselect(self, name: str, choices: list, default_values: list, label: str = None):
        if None in choices:
            raise Exception("Value None cannot be used as choice, use empty string instead")

        if type(default_values) != list:
            raise Exception("You must provide a list of length 1 if you want to specify default value")

        if len(default_values) != 1:
            raise Exception("Default values must contain exactly 1 element")

        label = self.__label_generator.generate_widget_label(name) if label is None else label
        default_value = str(default_values[0])

        self.__multiselect_fields.append(name)
        self.__dbutils.widgets.multiselect(name, default_value, choices, label)

    def remove(self, name: str):
        self.__dbutils.widgets.remove(name)
        self.__label_generator.remove(name)

    def remove_all(self):
        self.__dbutils.widgets.removeAll()
        self.__label_generator.remove_all()

    def get_value(self, name: str):
        value = self.__dbutils.widgets.get(name)

        if name in self.__multiselect_fields:
            if value == "":
                return []

            return value.split(",")

        return value

    def should_be_resolved(self):
        return is_databricks()



import re
from typing import List
from box import Box
from pyspark.sql.session import SparkSession
from consolebundle.detector import is_running_in_console
from injecta.container.ContainerInterface import ContainerInterface
from injecta.dtype.DType import DType
from injecta.service.Service import Service
from injecta.service.ServiceAlias import ServiceAlias
from injecta.service.argument.ServiceArgument import ServiceArgument
from pyfonybundles.Bundle import Bundle
from databricksbundle.notebook.NotebookErrorHandler import set_notebook_error_handler
from databricksbundle.detector import is_databricks
from databricksbundle.notebook.GithubLinkGenerator import GithubLinkGenerator
from databricksbundle.notebook.helpers import get_notebook_path, is_notebook_environment
from databricksbundle.notebook.logger.NotebookLoggerFactory import NotebookLoggerFactory


class DatabricksBundle(Bundle):

    DATABRICKS_NOTEBOOK = "databricks_notebook.yaml"
    DATABRICKS_SCRIPT = "databricks_script.yaml"
    DATABRICKS_CONNECT = "databricks_connect.yaml"

    @staticmethod
    def autodetect():
        if is_databricks():
            if is_notebook_environment():
                return DatabricksBundle(DatabricksBundle.DATABRICKS_NOTEBOOK)

            return DatabricksBundle(DatabricksBundle.DATABRICKS_SCRIPT)

        return DatabricksBundle(DatabricksBundle.DATABRICKS_CONNECT)

    def __init__(self, databricks_config: str):
        self.__databricks_config = databricks_config

    def get_config_files(self):
        return ["config.yaml", "databricks/" + self.__databricks_config]

    def modify_services(self, services: List[Service], aliases: List[ServiceAlias], parameters: Box):
        if is_running_in_console():
            aliases.append(ServiceAlias("databricksbundle.logger", "consolebundle.logger"))
        else:
            service = Service("databricksbundle.logger", DType("logging", "Logger"))
            service.set_factory(ServiceArgument(NotebookLoggerFactory.__module__), "create")

            services.append(service)

        return services, aliases

    def modify_parameters(self, parameters: Box) -> Box:
        if parameters.daipecore.logger.type == "default":
            parameters.daipecore.logger.type = "databricks"

        if is_databricks():
            parameters.pysparkbundle.dataframe.show_method = "databricks_display"
            parameters.daipecore.pandas.dataframe.show_method = "databricks_display"

        if parameters.pysparkbundle.filesystem is not None:
            raise Exception(
                "pysparkbundle.filesystem parameter must not be explicitly set as dbutils.fs must be used for Databricks-based projects"
            )

        parameters.pysparkbundle.filesystem = "dbutils.fs"

        return parameters

    def boot(self, container: ContainerInterface):
        parameters = container.get_parameters()

        if (
            is_databricks()
            and is_notebook_environment()
            and parameters.databricksbundle.enable_notebook_error_handler is True
            and not re.match("^/Users/", get_notebook_path())
        ):
            import IPython

            logger = container.get("databricksbundle.logger")

            set_notebook_error_handler(logger)

            multiple_results_enabled = "spark.databricks.workspace.multipleResults.enabled"

            spark = container.get(SparkSession)

            link_generator = GithubLinkGenerator()
            IPython.get_ipython().user_ns["daipe_help"] = link_generator.generate_link_from_module

            if spark.conf.get(multiple_results_enabled) == "false":
                logger.warning(f"{multiple_results_enabled} is set to false!")
                logger.warning("Error messages will not show properly!")



def resolve_dbutils():
    import IPython

    ipython = IPython.get_ipython()

    if not hasattr(ipython, "user_ns") or "dbutils" not in ipython.user_ns:
        raise Exception("dbutils cannot be resolved")

    return ipython.user_ns["dbutils"]



import unittest
from databricksbundle.widgets.DatabricksWidgets import DatabricksWidgets
from databricksbundle.widgets.testing import DbUtilsTesting, TestingDbUtilsWidgets


def set_input(**kwargs):
    TestingDbUtilsWidgets.set_raw_values(kwargs)


class DatabricksWidgetsTest(unittest.TestCase):
    def setUp(self):
        self.__widgets = DatabricksWidgets(DbUtilsTesting())

    # widgets.add_text() -----------------------

    def test_text(self):
        self.__widgets.add_text("mytext")

        set_input(mytext="January, February")
        self.assertEqual("January, February", self.__widgets.get_value("mytext"))

    def test_undefined_field(self):
        self.__widgets.add_text("mytext")
        set_input(mytext="January, February")

        with self.assertRaises(Exception) as error:
            self.__widgets.get_value("undefined_field")

        self.assertEqual('No widget defined for name "undefined_field"', str(error.exception))

    def test_text_default(self):
        self.__widgets.add_text("mytext", "my_default_value")
        set_input()
        self.assertEqual("my_default_value", self.__widgets.get_value("mytext"))

    def test_text_default_empty_string(self):
        self.__widgets.add_text("mytext", "")
        set_input()
        self.assertEqual("", self.__widgets.get_value("mytext"))

    def test_text_default_none(self):
        self.__widgets.add_text("mytext")
        set_input()

        self.assertEqual("", self.__widgets.get_value("mytext"))

    def test_text_default_override(self):
        self.__widgets.add_text("mytext", "my_default_value")
        set_input(mytext="January, February")
        self.assertEqual("January, February", self.__widgets.get_value("mytext"))

    def test_text_wrong_name(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_text("mytexť%")

        self.assertEqual(
            "The name you provided is incorrect, please provide name containing only alpha-numeric letters and _", str(error.exception)
        )

    def test_text_label_generation(self):
        self.__widgets.add_text("mytext")
        self.__widgets.add_text("yourtext")

        self.assertEqual("01. mytext", TestingDbUtilsWidgets.get_widget("mytext").label)

        self.assertEqual("02. yourtext", TestingDbUtilsWidgets.get_widget("yourtext").label)

    # widgets.add_select() -----------------------

    def test_select_with_default(self):
        self.__widgets.add_select("myselect", ["month", "year"], "year")
        set_input()
        self.assertEqual("year", self.__widgets.get_value("myselect"))

    def test_select_default_override(self):
        self.__widgets.add_select("myselect", ["month", "year"], "year")
        set_input(myselect="month")
        self.assertEqual("month", self.__widgets.get_value("myselect"))

    def test_select_no_input_default_empty_string(self):
        self.__widgets.add_select("myselect", ["month", "year", ""], "")
        set_input()
        self.assertEqual("", self.__widgets.get_value("myselect"))

    def test_select_with_none(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_select("myselect", ["month", "year", None], "")

        self.assertEqual("Value None cannot be used as choice, use empty string instead", str(error.exception))

    def test_select_default_not_in_choices(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_select("myselect", ["month", "year"], "")

        self.assertEqual('Default value "" not among choices', str(error.exception))

    def test_select_invalid_value(self):
        self.__widgets.add_select("myselect", ["month", "year"], "month")
        set_input(myselect="my_invalid_value")

        with self.assertRaises(Exception) as error:
            self.__widgets.get_value("myselect")

        self.assertEqual("argument --myselect: invalid choice: 'my_invalid_value' (choose from 'month', 'year')", str(error.exception))

    def test_select_wrong_name(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_select("myselect%", ["month", "year"], "month")

        self.assertEqual(
            "The name you provided is incorrect, please provide name containing only alpha-numeric letters and _", str(error.exception)
        )

    def test_select_label_generation(self):
        self.__widgets.add_select("myselect", ["month", "year"], "month")
        self.__widgets.add_select("yourselect", ["month", "year"], "month")

        self.assertEqual("01. myselect", TestingDbUtilsWidgets.get_widget("myselect").label)

        self.assertEqual("02. yourselect", TestingDbUtilsWidgets.get_widget("yourselect").label)

    # widgets.add_multiselect() -----------------------

    def test_multiselect_defaults(self):
        self.__widgets.add_multiselect("mymulti", ["January", "February", "March"], ["January"])
        set_input()
        self.assertEqual(["January"], self.__widgets.get_value("mymulti"))

    def test_multiselect_defaults_overridden(self):
        self.__widgets.add_multiselect("mymulti", ["January", "February", "March"], ["January"])
        set_input(mymulti=["January", "March"])
        self.assertEqual(["January", "March"], self.__widgets.get_value("mymulti"))

    def test_multiselect_defaults_empty(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_multiselect("mymulti", ["January", "February", "March"], [])

        self.assertEqual("Default values must contain exactly 1 element", str(error.exception))

    def test_multiselect_multi_values_one_invalid(self):
        self.__widgets.add_multiselect("mymulti", ["January", "February", "March"], ["March"])
        set_input(mymulti=["January", "April"])

        with self.assertRaises(Exception) as error:
            self.__widgets.get_value("mymulti")

        self.assertEqual("argument --mymulti: invalid choice: 'April' (choose from 'January', 'February', 'March')", str(error.exception))

    def test_multiselect_single_value_invalid(self):
        self.__widgets.add_multiselect("mymulti", ["January", "February", "March"], ["April"])
        set_input(mymulti=["April"])

        with self.assertRaises(Exception) as error:
            self.__widgets.get_value("mymulti")

        self.assertEqual(
            "argument --mymulti: invalid choice: 'April' (choose from 'January', 'February', 'March')",
            str(error.exception),
        )

    def test_multiselect_label_wrong_name(self):
        with self.assertRaises(Exception) as error:
            self.__widgets.add_multiselect("mymulti~", ["January", "February", "March"], ["January"])

        self.assertEqual(
            "The name you provided is incorrect, please provide name containing only alpha-numeric letters and _",
            str(error.exception),
        )

    def test_multiselect_label_generation(self):
        self.__widgets.add_multiselect("mymultiselect", ["January", "February", "March"], ["April"])
        self.__widgets.add_multiselect("yourmultiselect", ["January", "February", "March"], ["April"])

        self.assertEqual("01. mymultiselect", TestingDbUtilsWidgets.get_widget("mymultiselect").label)

        self.assertEqual("02. yourmultiselect", TestingDbUtilsWidgets.get_widget("yourmultiselect").label)


if __name__ == "__main__":
    unittest.main()



import tomlkit
import traceback

from inspect import getsource
from pathlib import Path


class GithubLinkGenerator:
    def __get_repo_name(self, module):
        special_repos = {"daipecore": "daipe-core", "featurestorebundle": "feature-store-bundle"}

        if module in special_repos.keys():
            return special_repos[module]

        return self.__get_repo_name_with_bundle(module)

    def __get_repo_name_with_bundle(self, module):
        index_of_bundle = module.find("bundle")

        return module[:index_of_bundle] + "-" + module[index_of_bundle:]

    def __get_module_version(self, module_name):
        base_path = Path("/".join(list(Path.cwd().parts[0:5])))
        lockfile_path = base_path.joinpath("poetry.lock")
        assert module_name == "databricks-bundle"
        with lockfile_path.open("r") as f:
            config = tomlkit.parse(f.read())

        for module in config["package"]:
            if module["name"] == module_name:
                return module["version"]

        raise Exception("Specified module could not be found, make sure the the module is imported and developed by DAIPE")

    def __get_display_html(self):
        import IPython

        ipython = IPython.get_ipython()

        if not hasattr(ipython, "user_ns") or "displayHTML" not in ipython.user_ns:
            raise Exception("displayHTML cannot be resolved")

        return ipython.user_ns["displayHTML"]

    def __get_github_url_from_module(self, module):
        module_path = module.__module__
        module_name_parent = module_path.split(".")[0]
        github_repo_name = self.__get_repo_name(module_name_parent)
        module_file_path = module_path.replace(".", "/") + ".py"
        version = self.__get_module_version(github_repo_name)

        return self.__generate__github_url(github_repo_name, version, module_file_path)

    def __generate__github_url(self, repo_name: str, version: str, file_path: str, lineno: str = ""):
        github_path = "https://github.com/daipe-ai/"

        return f"{github_path}{repo_name}/blob/v{version}/src/{file_path}#L{lineno}"

    def __get_file_path_from_stack_trace(self, stack_trace):
        filename_with_error = stack_trace.filename

        return filename_with_error.split("site-packages")[1]

    def generate_link_from_module(self, module):
        base_module_name = module.__module__.split(".")[-1]
        display_html = self.__get_display_html()
        module_github_url = self.__get_github_url_from_module(module)
        html_method_def = getsource(module).replace("\n", "<br>&nbsp;&nbsp;&nbsp;&nbsp;")
        html_link = f'<a href="{module_github_url}">Github source file for module {base_module_name}</a>'
        html_string = f"{html_method_def}<br>{html_link}"

        return display_html(html_string)

    def generate_link_from_traceback(self, tb):
        last_stack_trace = traceback.extract_tb(tb)[-1]
        file_path = self.__get_file_path_from_stack_trace(last_stack_trace)
        module_name = file_path.split("/")[1]
        github_repo_name = self.__get_repo_name_with_bundle(module_name)
        version = self.__get_module_version(github_repo_name)

        return self.__generate__github_url(github_repo_name, version, file_path, str(last_stack_trace.lineno))



from databricksbundle.notebook.GithubLinkGenerator import GithubLinkGenerator
from logging import Logger


# inspired by https://stackoverflow.com/questions/40110540/jupyter-magic-to-handle-notebook-exceptions
def set_notebook_error_handler(logger: Logger):  # noqa: 5302
    import IPython

    def custom_exc(shell, etype, evalue, tb, tb_offset=None):
        logger.error("Notebook exception", exc_info=True)

        logger.error(
            "The source file of the error can be found in the link below:\n" + GithubLinkGenerator().generate_link_from_traceback(tb)
        )
        # still show the error within the notebook, don't just swallow it
        shell.showtraceback((etype, evalue, tb), tb_offset=tb_offset)

    # this registers a custom exception handler for the whole current notebook
    IPython.get_ipython().set_custom_exc((Exception,), custom_exc)



from pyspark.sql.session import SparkSession
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface


class DictConfigConfigurator(ConfiguratorInterface):
    def __init__(
        self,
        dict_config: dict = None,
    ):
        self.__dict_config = dict_config or dict()

    def configure(self, spark: SparkSession):
        for k, v in self.__dict_config.items():
            spark.conf.set(k, v)



import re


class DatabricksWidgetsLabelGenerator(object):
    def __init__(self):
        self.__name_validator = re.compile("^[a-z][a-z_0-9]+$")
        self.__widget_labels = dict()
        self.__widget_index = 1

    def generate_widget_label(self, name):
        if self.__name_validator.match(name) is None:
            raise Exception("The name you provided is incorrect, please provide name containing only alpha-numeric letters and _")

        if name not in self.__widget_labels:
            self.__widget_labels[name] = self.__widget_index
            self.__widget_index += 1

        return f"{self.__widget_labels[name]:02d}. " + name.replace("_", " ")

    def remove(self, name: str):
        del self.__widget_labels[name]

    def remove_all(self):
        self.__widget_labels = dict()
        self.__widget_index = 1



import sys
from pathlib import Path
from databricksbundle.notebook.path.NotebookPathResolverInterface import NotebookPathResolverInterface


class LocalScriptPathResolver(NotebookPathResolverInterface):
    def resolve(self) -> Path:
        return Path(sys.argv[0])



from pyspark.sql import DataFrame
from pysparkbundle.dataframe.DataFrameShowMethodInterface import DataFrameShowMethodInterface
from databricksbundle.notebook.helpers import ipython_display


class DataFrameDisplay(DataFrameShowMethodInterface):
    def show(self, df: DataFrame):
        ipython_display(df)



from abc import ABC
from box import Box
from pyspark.sql.session import SparkSession


class StorageConfiguratorInterface(ABC):
    def configure(self, spark: SparkSession, config: Box):
        pass

    def get_type(self) -> str:
        pass



import unittest
from pyfonycore.bootstrap import bootstrapped_container
from injecta.testing.services_tester import test_services


class DatabricksBundleTest(unittest.TestCase):
    def test_azure(self):
        container = bootstrapped_container.init("test_azure")

        test_services(container)

    def test_aws(self):
        container = bootstrapped_container.init("test_aws")

        test_services(container)


if __name__ == "__main__":
    unittest.main()



from abc import ABC, abstractmethod
from typing import Dict

from pyspark.dbutils import DBUtils


class TestingWidget(ABC):
    @abstractmethod
    def get_default(self):
        pass

    @abstractmethod
    def get_value(self, raw_value):
        pass


class TextField(TestingWidget):
    def __init__(self, name: str, default_value: str = None, label: str = None):
        self.__name = name
        self.__default_value = default_value
        self.__label = label

    @property
    def name(self):
        return self.__name

    @property
    def default_value(self):
        return self.__default_value

    @property
    def label(self):
        return self.__label

    def get_default(self):
        return self.__default_value

    def get_value(self, raw_value):
        return raw_value


class Dropdown(TestingWidget):
    def __init__(self, name: str, choices: list, default_value: str = None, label: str = None):
        self.__name = name
        self.__choices = choices
        self.__default_value = default_value
        self.__label = label

    @property
    def name(self):
        return self.__name

    @property
    def choices(self):
        return self.__choices

    @property
    def default_value(self):
        return self.__default_value

    @property
    def label(self):
        return self.__label

    def get_default(self):
        return self.__default_value

    def get_value(self, raw_value):
        if raw_value not in self.__choices:
            choices_str = "'" + "', '".join(self.__choices) + "'"
            raise Exception(f"argument --{self.__name}: invalid choice: '{raw_value}' (choose from {choices_str})")

        return raw_value


class Multiselect(TestingWidget):
    def __init__(self, name: str, choices: list, default_values: list = None, label: str = None):
        self.__name = name
        self.__choices = choices
        self.__default_values = default_values or []
        self.__label = label

    @property
    def name(self):
        return self.__name

    @property
    def choices(self):
        return self.__choices

    @property
    def default_values(self):
        return self.__default_values

    @property
    def label(self):
        return self.__label

    def get_default(self):
        return self.default_values

    def get_value(self, raw_value):
        diff = set(raw_value) - set(self.__choices)

        if diff != set():
            choices_str = "'" + "', '".join(self.__choices) + "'"
            invalid_choices_str = "'" + "', '".join(diff) + "'"
            raise Exception(f"argument --{self.__name}: invalid choice: {invalid_choices_str} (choose from {choices_str})")

        return ",".join(raw_value)


class TestingDbUtilsWidgets:
    __fields = []
    _raw_values = dict()

    @classmethod
    def get_widget(cls, name):

        return next(field for field in cls.__fields if field.name == name)

    @classmethod
    def set_raw_values(cls, values: dict):
        cls._raw_values = values

    def text(self, name, default_value: str = "", label: str = None):
        if default_value is None:
            raise Exception("Default value cannot be None")

        self.__fields.append(TextField(name, default_value, label))

    def dropdown(self, name, default_value: str, choices: list, label: str = None):
        if default_value is None:
            raise Exception("Default value cannot be None")

        self.__fields.append(Dropdown(name, choices, default_value, label))

    def multiselect(self, name, default_values: list, choices: list, label: str = None):
        self.__fields.append(Multiselect(name, choices, default_values, label))

    def get(self, name):
        mapped_fields: Dict[str, TestingWidget] = {field.name: field for field in self.__fields}

        if name not in mapped_fields:
            raise Exception('No widget defined for name "undefined_field"')

        testing_widget = mapped_fields[name]

        if name not in TestingDbUtilsWidgets._raw_values:
            return testing_widget.get_default()

        return testing_widget.get_value(TestingDbUtilsWidgets._raw_values[name])


class DbUtilsTesting(DBUtils):
    def __init__(self):
        self.widgets = TestingDbUtilsWidgets()



import types
import platform
from databricksbundle.detector import is_databricks


class DbUtilsWrapper:
    def __init__(self, factory_callback: callable):
        self._factory_callback = factory_callback
        self._db_utils = None

    def __getattr__(self, attribute_name):
        if self._db_utils is None:
            if not is_databricks():
                from databricksbundle.spark.java_home_setter import set_java_home
                from databricksbundle.spark.existing_spark_cleanup import (
                    clean_existing_spark_config,
                )

                set_java_home()
                clean_existing_spark_config()

                if platform.system() == "Windows":
                    from databricksbundle.spark.hadoop_home_setter import (
                        set_hadoop_home_env_var,
                    )

                    set_hadoop_home_env_var()

            self._db_utils = self._factory_callback()

        if hasattr(self._db_utils, attribute_name) is False:
            raise AttributeError(attribute_name)

        attr = getattr(self._db_utils, attribute_name)

        if isinstance(attr, types.FunctionType) is False:
            return attr

        def wrapper(*args, **kw):
            return attr(*args, **kw)

        return wrapper



import json
import sys
from pyfonycore.bootstrap.config import config_factory as pyfony_config_factory
from pyfonycore.bootstrap.config.Config import Config
from databricksbundle.detector import is_databricks, is_databricks_repo

if sys.version_info >= (3, 8):
    from importlib import metadata as importlib_metadata
else:
    import importlib_metadata

from importlib_metadata import files


def read() -> Config:
    if not is_databricks() or is_databricks_repo():
        from pyfonycore.bootstrap.config import config_reader

        return config_reader.read()

    entry_points = importlib_metadata.entry_points().get("pyfony.bootstrap", ())

    if not entry_points:
        raise Exception("pyfony.bootstrap entry points is missing in the master package, try rebuilding the package")

    if hasattr(tuple(entry_points)[0], "dist"):
        dist_path = tuple(entry_points)[0].dist
    else:
        package_name = entry_points.value
        dist_path = files(package_name)[0].dist

    raw_config = json.loads(dist_path.read_text("bootstrap_config.json"))

    return pyfony_config_factory.create(raw_config, "pyfony.bootstrap entry point")


def entry_point_exists():
    return "pyfony.bootstrap" in importlib_metadata.entry_points()



import unittest
from databricksbundle.dbutils.DbUtilsMock import DbUtilsMock
from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper


class DbUtilsWrapperTest(unittest.TestCase):
    def test_object_attribute_delegation(self):
        def create_lazy():
            return DbUtilsMock()

        db_utils_wrapper = DbUtilsWrapper(create_lazy)
        result = db_utils_wrapper.fs.ls("/")

        self.assertIsInstance(result, list)

    def test_method_with_argument(self):
        def create_lazy():
            from pyspark.dbutils import FSHandler

            return FSHandler("foo")

        db_utils_wrapper = DbUtilsWrapper(create_lazy)
        result = db_utils_wrapper.print_return(12345)

        self.assertEqual(12345, result)

    def test_delegation_non_existent_attribute(self):
        def create_lazy():
            return DbUtilsMock()

        db_utils_wrapper = DbUtilsWrapper(create_lazy)

        with self.assertRaises(AttributeError):
            db_utils_wrapper.non_existent_method()


if __name__ == "__main__":
    unittest.main()



from logging import Logger
from pyspark.sql import SparkSession
from argparse import Namespace
from consolebundle.ConsoleCommand import ConsoleCommand


class DatabricksConnectTestCommand(ConsoleCommand):
    def __init__(self, logger: Logger, spark: SparkSession):
        self.__logger = logger
        self.__spark = spark

    def get_command(self) -> str:
        return "dbx:test-connection"

    def get_description(self):
        return "Test databricks-connect connection to Databricks instance"

    def run(self, input_args: Namespace):
        self.__logger.info("Testing the connectivity")
        a = [1, 2, 3, 4]
        b = [2, 3, 4, 8]
        df = self.__spark.createDataFrame([a, b], schema=["a", "b"])
        self.__logger.info("Creating sample DataFrame")
        df.show()
        self.__logger.info("Connection successful")



import sys
from pathlib import Path
from databricksbundle.notebook.path.NotebookPathResolverInterface import NotebookPathResolverInterface


class DatabricksScriptPathResolver(NotebookPathResolverInterface):
    def resolve(self) -> Path:
        if len(sys.argv) == 1:
            raise Exception(
                "spark_python_task.parameters in Databricks job configuration must contain real pipeline path as the first parameter"
            )

        return Path(sys.argv[1])



from pyspark.sql import SparkSession
from databricksbundle.dbutils.DbUtilsWrapper import DbUtilsWrapper


class DatabricksConnectDbUtilsFactory:
    def __init__(self, spark: SparkSession):
        self.__spark = spark

    def create(self) -> DbUtilsWrapper:
        def create_lazy():
            from pyspark.dbutils import DBUtils

            return DBUtils(self.__spark)

        return DbUtilsWrapper(create_lazy)



from box import Box
from typing import List, Dict
from pyspark.sql.session import SparkSession
from databricksbundle.storage.StorageConfiguratorInterface import StorageConfiguratorInterface
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface


class StorageConfigurator(ConfiguratorInterface):

    __storage_configurators: Dict[str, StorageConfiguratorInterface]

    def __init__(self, storages: Box, storage_configurators: List[StorageConfiguratorInterface]):
        self.__storages = storages or Box({})
        self.__storage_configurators = {
            storage_configurator.get_type(): storage_configurator for storage_configurator in storage_configurators
        }

    def configure(self, spark: SparkSession):
        for key, conf in self.__storages.items():
            if conf.type not in self.__storage_configurators:
                raise Exception(f"No configurator for storage: {conf.type}")

            storage_configurator = self.__storage_configurators[conf.type]

            storage_configurator.configure(spark, conf)



from typing import List
from pyspark.sql import SparkSession
from databricksbundle.spark.SparkSessionLazy import SparkSessionLazy
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface


class NotebookSessionFactory:
    def __init__(
        self,
        configurators: List[ConfiguratorInterface],
    ):
        self.__configurators = configurators

    def create(self) -> SparkSessionLazy:
        import IPython

        spark: SparkSession = IPython.get_ipython().user_ns["spark"]

        for configurator in self.__configurators:
            configurator.configure(spark)

        IPython.get_ipython().user_ns["spark"] = spark

        return SparkSessionLazy(lambda: spark)



from box import Box
from pyspark.sql.session import SparkSession
from pyspark.dbutils import DBUtils
from databricksbundle.storage.StorageConfiguratorInterface import StorageConfiguratorInterface


class AzureGen2Configurator(StorageConfiguratorInterface):
    def configure(self, spark: SparkSession, config: Box):
        # initialized manually to prevent issues with Databricks Connect initialization
        dbutils = DBUtils(spark)

        spark.conf.set(
            f"fs.azure.account.oauth2.client.endpoint.{config.storage_name}.dfs.core.windows.net",
            f"https://login.microsoftonline.com/{config.tenant_id}/oauth2/token",
        )
        spark.conf.set(
            f"fs.azure.account.oauth.provider.type.{config.storage_name}.dfs.core.windows.net",
            "org.apache.hadoop.fs.azurebfs.oauth2.ClientCredsTokenProvider",
        )
        spark.conf.set(f"fs.azure.account.auth.type.{config.storage_name}.dfs.core.windows.net", "OAuth")
        spark.conf.set(
            f"fs.azure.account.oauth2.client.id.{config.storage_name}.dfs.core.windows.net",
            dbutils.secrets.get(scope=config.client_id.secret_scope, key=config.client_id.secret_key),
        )
        spark.conf.set(
            f"fs.azure.account.oauth2.client.secret.{config.storage_name}.dfs.core.windows.net",
            dbutils.secrets.get(scope=config.client_secret.secret_scope, key=config.client_secret.secret_key),
        )

    def get_type(self):
        return "azure_gen2"



import unittest
from pyfonycore.bootstrap import bootstrapped_container
from databricksbundle.storage.StorageConfigurator import StorageConfigurator
from databricksbundle.storage.testing.SparkSessionMock import SparkSessionMock


class StorageConfiguratorTest(unittest.TestCase):
    def test_azure(self):
        container = bootstrapped_container.init("test_azure")
        storage_configurator: StorageConfigurator = container.get(StorageConfigurator)

        spark_mock = SparkSessionMock()

        storage_configurator.configure(spark_mock)

        self.assertEqual("tenant/123456", spark_mock.conf.get("testing.storage.aaa"))
        self.assertEqual("secrets/some_client_id_scope1/some_client_id_key1", spark_mock.conf.get("testing.secrets.aaa.client_id"))
        self.assertEqual(
            "secrets/some_client_secret_scope1/some_client_secret_key1", spark_mock.conf.get("testing.secrets.aaa.client_secret")
        )

        self.assertEqual("tenant/987654", spark_mock.conf.get("testing.storage.bbb"))
        self.assertEqual("secrets/some_client_id_scope2/some_client_id_key2", spark_mock.conf.get("testing.secrets.bbb.client_id"))
        self.assertEqual(
            "secrets/some_client_secret_scope2/some_client_secret_key2", spark_mock.conf.get("testing.secrets.bbb.client_secret")
        )


if __name__ == "__main__":
    unittest.main()



import os


def is_databricks():
    """
    Check, if scripts are running on databricks cluster or locally.

    DATABRICKS_RUNTIME_VERSION available for all version and only on databricks cluster
    :return: true, if running on databricks
    """
    return os.getenv("DATABRICKS_RUNTIME_VERSION") is not None


def is_databricks_repo():
    return is_databricks() and os.getcwd().startswith("/Workspace/Repos")



import sys
from databricksbundle.dbutils.IPythonDbUtilsResolver import resolve_dbutils


def get_notebook_context():
    return resolve_dbutils().notebook.entry_point.getDbutils().notebook().getContext()


def get_user_email():
    return get_notebook_context().tags().get("user").get()


def get_notebook_path():
    return get_notebook_context().notebookPath().get()


def is_notebook_environment():
    return sys.argv and sys.argv[0][-15:] == "/PythonShell.py"


def ipython_display(obj):
    import IPython

    IPython.get_ipython().user_ns["display"](obj)



from logging import Logger
from loggerbundle.LoggerFactory import LoggerFactory
from databricksbundle.notebook.path.NotebookPathResolverInterface import NotebookPathResolverInterface


class NotebookLoggerFactory:
    def __init__(
        self,
        notebook_path_resolver: NotebookPathResolverInterface,
        logger_factory: LoggerFactory,
    ):
        self.__notebook_path_resolver = notebook_path_resolver
        self.__logger_factory = logger_factory

    def create(self) -> Logger:
        notebook_path = self.__notebook_path_resolver.resolve()

        return self.__logger_factory.create(notebook_path.parent.stem)



from pyspark.dbutils import DBUtils
from pysparkbundle.filesystem.FilesystemInterface import FilesystemInterface


class DbUtilsFilesystem(FilesystemInterface):
    def __init__(
        self,
        dbutils: DBUtils,
    ):
        self.__dbutils = dbutils

    def exists(self, path: str):
        try:
            self.__dbutils.fs.head(path)

            return True
        except Exception as e:
            if "Cannot head a directory:" in str(e):
                return True

            if "java.io.FileNotFoundException" in str(e):
                return False

            raise

    def put(self, path: str, content: str, overwrite: bool = False):
        self.__dbutils.fs.put(path, content, overwrite)

    def makedirs(self, path: str):
        self.__dbutils.fs.mkdirs(path)

    def copy(self, source: str, destination: str, recursive: bool = False):
        self.__dbutils.fs.cp(source, destination, recursive)

    def move(self, source: str, destination: str, recursive: bool = False):
        self.__dbutils.fs.mv(source, destination, recursive)

    def delete(self, path: str, recursive: bool = False):
        self.__dbutils.fs.rm(path, recursive)



from pyspark.dbutils import FileInfo


class FSHandlerMock:
    def ls(self, path: str):
        return [
            FileInfo("/foo/bar", "something.txt", 123),
            FileInfo("/foo/bar", "something_else.txt", 789),
        ]


class DbUtilsMock:
    def __init__(self):
        self.fs = FSHandlerMock()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['databricksbundle',
 'databricksbundle.bootstrap',
 'databricksbundle.dbutils',
 'databricksbundle.filesystem',
 'databricksbundle.notebook',
 'databricksbundle.notebook.logger',
 'databricksbundle.notebook.path',
 'databricksbundle.spark',
 'databricksbundle.spark.config',
 'databricksbundle.storage',
 'databricksbundle.storage.adapter',
 'databricksbundle.storage.testing',
 'databricksbundle.test',
 'databricksbundle.widgets']

package_data = \
{'': ['*'], 'databricksbundle': ['_config/*', '_config/databricks/*']}

install_requires = \
['console-bundle>=0.5,<0.6',
 'daipe-core>=1.2,<2.0',
 'injecta>=0.10.0,<0.11.0',
 'logger-bundle>=0.7.0,<0.8.0',
 'pyfony-bundles>=0.4.0,<0.5.0',
 'pyfony-core>=0.8.0,<0.9.0',
 'pyspark-bundle>=1.2,<2.0']

entry_points = \
{'daipe': ['bootstrap_config_reader = '
           'databricksbundle.bootstrap.package_config_reader:read'],
 'pyfony.bundle': ['create = '
                   'databricksbundle.DatabricksBundle:DatabricksBundle.autodetect']}

setup_kwargs = {
    'name': 'databricks-bundle',
    'version': '1.4.2.dev12',
    'description': 'Databricks runtime support for the Daipe framework',
    'long_description': '# Databricks bundle\n\nThis bundle allows you use [Daipe Framework](https://www.daipe.ai) with Databricks.  \n\n## Resources\n\n* [Documentation](https://docs.daipe.ai/data-pipelines-workflow/coding-basics/)\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daipe-ai/databricks-bundle',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import List
from box import Box
from pyspark.sql.session import SparkSession
from pyspark.conf import SparkConf
from databricksbundle.spark.SparkSessionLazy import SparkSessionLazy
from databricksbundle.spark.config.ConfiguratorInterface import ConfiguratorInterface


class DatabricksConnectSessionFactory:
    def __init__(
        self,
        config: Box,
        configurators: List[ConfiguratorInterface],
    ):
        self.__config = config
        self.__configurators = configurators

    def create(self) -> SparkSessionLazy:
        def create_lazy():
            if not self.__config.address:
                raise Exception("Databricks workspace address not set")

            if not self.__config.token:
                raise Exception("Databricks workspace token not set")

            if not self.__config.cluster_id:
                raise Exception("Databricks cluster not set")

            if not self.__config.port:
                raise Exception("Databricks connect port not set")

            # Databricks Connect configuration must be set before calling getOrCreate()
            conf = SparkConf()
            conf.set("spark.databricks.service.address", self.__config.address)
            conf.set("spark.databricks.service.token", self.__config.token)
            conf.set("spark.databricks.service.clusterId", self.__config.cluster_id)

            if self.__config.org_id is not None:
                conf.set("spark.databricks.service.orgId", self.__config.org_id)

            conf.set("spark.databricks.service.port", self.__config.port)

            if self.__config.driver_bind_address is not None:
                conf.set("spark.driver.bindAddress", self.__config.driver_bind_address)

            spark = SparkSession.builder.config(conf=conf).getOrCreate()

            for configurator in self.__configurators:
                configurator.configure(spark)

            return spark

        return SparkSessionLazy(create_lazy)



from pathlib import Path
import os


def set_hadoop_home_env_var():
    venv_path = os.getcwd() + "/.venv"

    hadoop_home_path = Path(Path(venv_path).joinpath("hadoop"))

    os.environ["HADOOP_HOME"] = str(hadoop_home_path)



from pathlib import Path
from databricksbundle.notebook.helpers import get_notebook_path
from databricksbundle.notebook.path.NotebookPathResolverInterface import NotebookPathResolverInterface


class DatabricksNotebookPathResolver(NotebookPathResolverInterface):
    def resolve(self) -> Path:
        return Path(get_notebook_path())



class SparkConf:

    vals: dict = {}

    def set(self, key: str, val: str):
        self.vals[key] = val

    def get(self, key: str):
        if key not in self.vals:
            raise Exception(f"Conf value not found for {key}")

        return self.vals[key]


class SparkSessionMock:
    def __init__(self):
        self.conf = SparkConf()
